'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const pandora_dollar_1 = require("pandora-dollar");
const const_1 = require("../const");
const LoggerBroker_1 = require("./LoggerBroker");
const extend = require("extend");
class GlobalConfigProcessor {
    constructor() {
        this.globalConfig = {};
        this.initialized = false;
        this.loadedConfigPath = new Set();
    }
    static getInstance() {
        if (!this.instance) {
            this.instance = new GlobalConfigProcessor();
        }
        return this.instance;
    }
    getAllProperties() {
        if (!this.initialized) {
            this.initialized = true;
            const cwd = process.env[const_1.PANDORA_CWD] || process.cwd();
            const cwdRequire = pandora_dollar_1.makeRequire(cwd);
            const configPaths = process.env[const_1.PANDORA_GLOBAL_CONFIG] ? process.env[const_1.PANDORA_GLOBAL_CONFIG].split(GlobalConfigProcessor.GLOBAL_PACKAGE_SPLIT) : [];
            // set default config first
            configPaths.unshift(require.resolve('../default'));
            for (const configPath of configPaths) {
                if (configPath && configPath !== 'undefined' && !this.loadedConfigPath.has(configPath)) {
                    try {
                        // require module maybe async method
                        // The CompilerDispatcher uses a combination of idle tasks and background tasks to parse and compile lazily parsed functions.
                        this.loadedConfigPath.add(configPath);
                        let extendConfig = cwdRequire(configPath);
                        extendConfig = extendConfig.default ? extendConfig.default : extendConfig;
                        this.mergeProperties(extendConfig);
                    }
                    catch (err) {
                        // info
                        this.loadedConfigPath.delete(configPath);
                        LoggerBroker_1.consoleLogger.info(`Can't find config from ${configPath}`);
                    }
                }
            }
        }
        return this.globalConfig;
    }
    // merge other properties after init
    mergeProperties(properties) {
        try {
            this.globalConfig = extend(true, this.globalConfig, properties);
        }
        catch (err) {
            // info
            LoggerBroker_1.consoleLogger.error(`merge global config error`, err);
        }
    }
    getloadedConfigPath() {
        return Array.from(this.loadedConfigPath);
    }
    // for test
    clearProperties() {
        this.globalConfig = {};
        this.initialized = false;
        this.loadedConfigPath.clear();
    }
}
GlobalConfigProcessor.GLOBAL_PACKAGE_SPLIT = ':';
exports.GlobalConfigProcessor = GlobalConfigProcessor;
//# sourceMappingURL=GlobalConfigProcessor.js.map